@@ -44,10 +44,12 @@ __all__ = [
     'ClassDeclaration',
     'CompilationUnit',
     'Continue',
+    'DeconstructionPattern',
     'DoWhileLoop',
     'Empty',
     'EnumValue',
     'EnumValueSet',
+    'Erroneous',
     'FieldAccess',
     'ForEachLoop',
     'ForLoop',
@@ -89,5 +91,9 @@ __all__ = [
     'WhileLoop',
     'Wildcard',
     'Yield',
-    'Unknown'
+    'Unknown',
+
+    # Templating
+    'CoordinateBuilder',
+    'JavaCoordinates',
 ]
