@@ -1,9 +1,10 @@
 from __future__ import annotations
 
 import weakref
+from abc import ABC, abstractmethod
 from dataclasses import dataclass, replace
 from pathlib import Path
-from typing import List, Optional, Protocol, runtime_checkable, TYPE_CHECKING
+from typing import List, Optional, TYPE_CHECKING
 from uuid import UUID
 from enum import Enum
 
@@ -95,13 +96,13 @@ class Await(Py, Expression):
     def with_expression(self, expression: Expression) -> Await:
         return self if expression is self._expression else replace(self, _expression=expression)
 
-    _type: JavaType
+    _type: Optional[JavaType]
 
     @property
-    def type(self) -> JavaType:
+    def type(self) -> Optional[JavaType]:
         return self._type
 
-    def with_type(self, type: JavaType) -> Await:
+    def with_type(self, type: Optional[JavaType]) -> Await:
         return self if type is self._type else replace(self, _type=type)
 
     def accept_python(self, v: PythonVisitor[P], p: P) -> J:
@@ -339,13 +340,13 @@ class ExceptionType(Py, TypeTree):
     def with_markers(self, markers: Markers) -> ExceptionType:
         return self if markers is self._markers else replace(self, _markers=markers)
 
-    _type: JavaType
+    _type: Optional[JavaType]
 
     @property
-    def type(self) -> JavaType:
+    def type(self) -> Optional[JavaType]:
         return self._type
 
-    def with_type(self, type: JavaType) -> ExceptionType:
+    def with_type(self, type: Optional[JavaType]) -> ExceptionType:
         return self if type is self._type else replace(self, _type=type)
 
     _exception_group: bool
@@ -502,13 +503,13 @@ class LiteralType(Py, Expression, TypeTree):
     def with_literal(self, literal: Expression) -> LiteralType:
         return self if literal is self._literal else replace(self, _literal=literal)
 
-    _type: JavaType
+    _type: Optional[JavaType]
 
     @property
-    def type(self) -> JavaType:
+    def type(self) -> Optional[JavaType]:
         return self._type
 
-    def with_type(self, type: JavaType) -> LiteralType:
+    def with_type(self, type: Optional[JavaType]) -> LiteralType:
         return self if type is self._type else replace(self, _type=type)
 
     def accept_python(self, v: PythonVisitor[P], p: P) -> J:
@@ -553,13 +554,13 @@ class TypeHint(Py, TypeTree):
     def with_type_tree(self, type_tree: Expression) -> TypeHint:
         return self if type_tree is self._type_tree else replace(self, _type_tree=type_tree)
 
-    _type: JavaType
+    _type: Optional[JavaType]
 
     @property
-    def type(self) -> JavaType:
+    def type(self) -> Optional[JavaType]:
         return self._type
 
-    def with_type(self, type: JavaType) -> TypeHint:
+    def with_type(self, type: Optional[JavaType]) -> TypeHint:
         return self if type is self._type else replace(self, _type=type)
 
     def accept_python(self, v: PythonVisitor[P], p: P) -> J:
@@ -702,7 +703,10 @@ class CompilationUnit(Py, JavaSourceFile, SourceFile):
         return p
 
     def printer(self, cursor: Cursor) -> TreeVisitor[Tree, PrintOutputCapture[P]]:
-        return PrinterFactory.current().create_printer(cursor)
+        if factory := PrinterFactory.current():
+            return factory.create_printer(cursor)
+        from .printer import PythonPrinter
+        return PythonPrinter[PrintOutputCapture[P]]()
 
     def accept_python(self, v: PythonVisitor[P], p: P) -> J:
         return v.visit_compilation_unit(self, p)
@@ -719,6 +723,20 @@ class ExpressionStatement(Py, Expression, Statement):
     def with_id(self, id: UUID) -> ExpressionStatement:
         return self if id is self._id else replace(self, _id=id)
 
+    @property
+    def prefix(self) -> Space:
+        return self._expression.prefix
+
+    def with_prefix(self, prefix: Space) -> ExpressionStatement:
+        return self.with_expression(self._expression.with_prefix(prefix))
+
+    @property
+    def markers(self) -> Markers:
+        return self._expression.markers
+
+    def with_markers(self, markers: Markers) -> ExpressionStatement:
+        return self.with_expression(self._expression.with_markers(markers))
+
     _expression: Expression
 
     @property
@@ -790,7 +808,14 @@ class StatementExpression(Py, Expression, Statement):
         return self._statement.prefix
 
     def with_prefix(self, prefix: Space) -> StatementExpression:
-        return self._statement.with_prefix(prefix)
+        return self.with_statement(self._statement.with_prefix(prefix))
+
+    @property
+    def markers(self) -> Markers:
+        return self._statement.markers
+
+    def with_markers(self, markers: Markers) -> StatementExpression:
+        return self.with_statement(self._statement.with_markers(markers))
 
     _statement: Statement
 
@@ -838,7 +863,7 @@ class MultiImport(Py, Statement):
 
     @property
     def from_(self) -> Optional[NameTree]:
-        return self._from.element
+        return self._from.element if self._from else None
 
     def with_from(self, from_: Optional[NameTree]) -> MultiImport:
         return self.padding.with_from(JRightPadded.with_element(self._from, from_))
@@ -1251,7 +1276,7 @@ class FormattedString(Py, Expression, TypedTree):
 
         @property
         def debug(self) -> Optional[bool]:
-            return self._debug.element
+            return self._debug.element if self._debug else None
 
         def with_debug(self, debug: Optional[bool]) -> FormattedString.Value:
             return self.padding.with_debug(JRightPadded.with_element(self._debug, debug))
@@ -1582,7 +1607,7 @@ class ComprehensionExpression(Py, Expression):
 
         @property
         def async_(self) -> Optional[bool]:
-            return self._async.element
+            return self._async.element if self._async else None
 
         def with_async(self, async_: Optional[bool]) -> ComprehensionExpression.Clause:
             return self.padding.with_async(JRightPadded.with_element(self._async, async_))
@@ -1780,13 +1805,13 @@ class YieldFrom(Py, Expression):
     def with_expression(self, expression: Expression) -> YieldFrom:
         return self if expression is self._expression else replace(self, _expression=expression)
 
-    _type: JavaType
+    _type: Optional[JavaType]
 
     @property
-    def type(self) -> JavaType:
+    def type(self) -> Optional[JavaType]:
         return self._type
 
-    def with_type(self, type: JavaType) -> YieldFrom:
+    def with_type(self, type: Optional[JavaType]) -> YieldFrom:
         return self if type is self._type else replace(self, _type=type)
 
     def accept_python(self, v: PythonVisitor[P], p: P) -> J:
@@ -2344,13 +2369,13 @@ class ErrorFrom(Py, Expression):
     def with_from(self, from_: Expression) -> ErrorFrom:
         return self.padding.with_from(JLeftPadded.with_element(self._from, from_))
 
-    _type: JavaType
+    _type: Optional[JavaType]
 
     @property
-    def type(self) -> JavaType:
+    def type(self) -> Optional[JavaType]:
         return self._type
 
-    def with_type(self, type: JavaType) -> ErrorFrom:
+    def with_type(self, type: Optional[JavaType]) -> ErrorFrom:
         return self if type is self._type else replace(self, _type=type)
 
     @dataclass
@@ -2426,7 +2451,7 @@ class MatchCase(Py, Expression):
 
     @property
     def guard(self) -> Optional[Expression]:
-        return self._guard.element
+        return self._guard.element if self._guard else None
 
     def with_guard(self, guard: Optional[Expression]) -> MatchCase:
         return self.padding.with_guard(JLeftPadded.with_element(self._guard, guard))
@@ -2609,7 +2634,7 @@ class Slice(Py, Expression, TypedTree):
 
     @property
     def start(self) -> Optional[Expression]:
-        return self._start.element
+        return self._start.element if self._start else None
 
     def with_start(self, start: Optional[Expression]) -> Slice:
         return self.padding.with_start(JRightPadded.with_element(self._start, start))
@@ -2618,7 +2643,7 @@ class Slice(Py, Expression, TypedTree):
 
     @property
     def stop(self) -> Optional[Expression]:
-        return self._stop.element
+        return self._stop.element if self._stop else None
 
     def with_stop(self, stop: Optional[Expression]) -> Slice:
         return self.padding.with_stop(JRightPadded.with_element(self._stop, stop))
@@ -2627,7 +2652,7 @@ class Slice(Py, Expression, TypedTree):
 
     @property
     def step(self) -> Optional[Expression]:
-        return self._step.element
+        return self._step.element if self._step else None
 
     def with_step(self, step: Optional[Expression]) -> Slice:
         return self.padding.with_step(JRightPadded.with_element(self._step, step))
